@@ -33,6 +33,10 @@ Crypt::OpenSSL::CA::Inline::C - A bag of XS and L<Inline::C> tricks
 
 =head1 DESCRIPTION
 
+B<This documentation is only useful for people who want to hack
+I<Crypt::OpenSSL::CA>. It is of no interest for people who just want
+to use the module.>
+
 This package provides L<Inline::C> goodness to L<Crypt::OpenSSL::CA>
 during development, plus a few tricks of our own.  The idiom in
 L</SYNOPSIS>, used throughout the source code of
@@ -72,8 +76,9 @@ the following C functions:
 =item I<static inline char* char0_value(SV* string)>
 
 Returns the string value of a Perl SV, making sure that it exists and
-is zero-terminated beforehand. See L<perlguts/Working with SVs>, look
-for the word "Nevertheless". I'm pretty sure there is a macro in
+is zero-terminated beforehand. If C<string> is undef, returns the
+empty string (B<not> NULL).  See L<perlguts/Working with SVs>, look
+for the word "Nevertheless" - I'm pretty sure there is a macro in
 Perl's convenience stuff that does exactly that already, but I don't
 know it...
 
@@ -166,6 +171,52 @@ where $formattedstring is the C<sprintf>-formatted version of the
 arguments passed to I<sslcroak>, and the OpenSSL error strings are
 retrieved using B<ERR_get_error(3)> and B<ERR_error_string(3)>.
 
+=item I<static ASN1_TIME* parse_RFC3280_time(char* datetime,
+              char** errmsg, char* sslerrmsg)>
+
+Parses C<datetime>, a date in "Zulu" format (that is, yyyymmddhhmmssZ,
+with a literal Z at the end), and returns a newly-allocated ASN1_TIME*
+structure utilizing a C<utcTime> encoding for dates in the year 2049
+or before and C<generalizedTime> for dates in 2050 and after.  RFC3280
+dictates that this convention should apply to most date-related fields
+in X509 certificates and CRLs (as per sections 4.1.2.5 for certificate
+validity periods, and 5.1.2.4 through 5.1.2.6 for CRL validity periods
+and certificate revocation times).  By contrast, the C<invalidityDate>
+CRL revocation reason extension is always in C<generalizedTime> and
+this function should not be used there.
+
+If there is an error, NULL is returned, and one (and only
+one) of *errmsg and *sslerrmsg is set to an error string, provided
+that they are not NULL.  Caller should thereafter call I<croak> or
+L</sslcroak> respectively.
+
+=item I<static ASN1_TIME* parse_RFC3280_time_or_croak(char* datetime)>
+
+Like L</parse_RFC3280_time> except that it handles its errors itself and
+will therefore never return NULL.  The caller should not have an
+outstanding temporary variable that must be freed before it returns,
+or a memory leak will be created; if this is the case, use the more
+clunky L</parse_serial> form instead.
+
+=item I<static ASN1_INTEGER* parse_serial
+              (char* hexserial, char** errmsg, char** sslerrmsg)>
+
+Parses hexserial, a lowercase, hexadecimal string that starts with
+"0x", and returns it as a newly-allocated C<ASN1_INTEGER> structure
+that must be freed by caller (with C<ASN1_INTEGER_free>) when done
+with it.  If there is an error, NULL is returned, and one (and only
+one) of *errmsg and *sslerrmsg is set to an error string, provided
+that they are not NULL.  Caller should thereafter call I<croak> or
+L</sslcroak> respectively.
+
+=item I<static ASN1_INTEGER* parse_serial_or_croak(char* hexserial)>
+
+Like L</parse_serial> except that it handles its errors itself and
+will therefore never return NULL.  The caller should not have an
+outstanding temporary variable that must be freed before it returns,
+or a memory leak will be created; if this is the case, use the more
+clunky L</parse_serial> form instead.
+
 =back
 
 =cut
@@ -180,8 +231,8 @@ sub _c_boilerplate { <<'C_BOILERPLATE'; }
 #include <openssl/bio.h>    /* Also for BIO_mem_to_SV() */
 
 #include <openssl/opensslv.h>
-#if OPENSSL_VERSION_NUMBER < 0x00905000
-#error OpenSSL version 0.9.5 is required in order to handle Unicode DNs.
+#if OPENSSL_VERSION_NUMBER < 0x00907000
+#error OpenSSL version 0.9.7 or later is required. See comments in CA.pm
 #endif
 
 static inline char* char0_value(SV* perlscalar) {
@@ -310,6 +361,91 @@ static void sslcroak(char *fmt, ...) {
     }
 }
 
+/* RFC3280, section 4.1.2.5 */
+#define RFC3280_cutoff_date "20500000" "000000"
+static ASN1_TIME* parse_RFC3280_time(char* date,
+                                     char** errmsg, char** sslerrmsg) {
+    int status;
+    int is_generalizedtime;
+    ASN1_TIME* retval;
+
+    if (strlen(date) != strlen(RFC3280_cutoff_date) + 1) {
+         if (errmsg) { *errmsg = "Wrong date length"; }
+         return NULL;
+    }
+    if (date[strlen(RFC3280_cutoff_date)] != 'Z') {
+         if (errmsg) { *errmsg = "Wrong date format"; }
+         return NULL;
+    }
+
+    if (! (retval = ASN1_TIME_new())) {
+         if (errmsg) { *errmsg = "ASN1_TIME_new failed"; }
+         return NULL;
+    }
+
+    is_generalizedtime = (strcmp(date, RFC3280_cutoff_date) > 0);
+    if (! (is_generalizedtime ?
+           ASN1_GENERALIZEDTIME_set_string(retval, date) :
+           ASN1_UTCTIME_set_string(retval, date + 2)) ) {
+        ASN1_TIME_free(retval);
+        if (errmsg) {
+            *errmsg = (is_generalizedtime ?
+               "ASN1_GENERALIZEDTIME_set_string failed (bad date format?)" :
+               "ASN1_UTCTIME_set_string failed (bad date format?)");
+        }
+        return NULL;
+    }
+    return retval;
+}
+
+static ASN1_TIME* parse_RFC3280_time_or_croak(char* date) {
+    char* plainerr = NULL; char* sslerr = NULL;
+    ASN1_INTEGER* retval = NULL;
+    if ((retval = parse_RFC3280_time(date, &plainerr, &sslerr))) {
+        return retval;
+    }
+    if (plainerr) { croak(plainerr); }
+    if (sslerr) { sslcroak(sslerr); }
+    croak("Unknown error in parse_RFC3280_time");
+    return NULL; /* Not reached */
+}
+
+static ASN1_INTEGER* parse_serial(char* hexserial,
+          char** errmsg, char** sslerrmsg) {
+    BIGNUM* serial = NULL;
+    ASN1_INTEGER* retval;
+
+    if (! (hexserial[0] == '0' && hexserial[1] == 'x')) {
+        if (errmsg) {
+            *errmsg = "Bad serial string, should start with 0x";
+        }
+        return NULL;
+    }
+    if (! BN_hex2bn(&serial, hexserial + 2)) {
+        if (sslerrmsg) { *sslerrmsg = "BN_hex2bn failed"; }
+        return NULL;
+    }
+    retval = BN_to_ASN1_INTEGER(serial, NULL);
+    BN_free(serial);
+    if (! retval) {
+        if (sslerrmsg) { *sslerrmsg = "BN_to_ASN1_INTEGER failed"; }
+        return NULL;
+    }
+    return retval;
+}
+
+static ASN1_INTEGER* parse_serial_or_croak(char* hexserial) {
+    char* plainerr = NULL; char* sslerr = NULL;
+    ASN1_INTEGER* retval = NULL;
+    if ((retval = parse_serial(hexserial, &plainerr, &sslerr))) {
+        return retval;
+    }
+    if (plainerr) { croak(plainerr); }
+    if (sslerr) { sslcroak(sslerr); }
+    croak("Unknown error in parse_serial");
+    return NULL; /* Not reached */
+}
+
 C_BOILERPLATE
 
 =head1 INTERNALS
@@ -317,8 +453,6 @@ C_BOILERPLATE
 The C<< use Crypt::OpenSSL::CA::Inline::C >> idiom described in
 L</SYNOPSIS> is implemented in terms of L<Inline>.
 
-DOCUMENTME
-
 =over
 
 =cut
@@ -332,8 +466,6 @@ MESSAGE
 
 use Inline::C ();
 
-my $debugging; BEGIN { $debugging = 1; } # FIXME: make this configurable
-
 =item I<import()>
 
 Called whenever one of the C<< use Crypt::OpenSSL::CA::Inline::C "foo"
@@ -371,11 +503,11 @@ sub import {
          NAME => $package,
          ( $Crypt::OpenSSL::CA::VERSION ?
            (VERSION => $Crypt::OpenSSL::CA::VERSION) : () ),
-         LIBS => join(" ", qw(-lcrypto -lssl),
-                      ($debugging ? qw(-g) : ())),
-         CCFLAGS => join(" ", qw(-Wall -Wno-unused -Werror),
-                         ($debugging ? qw(-g) : ())),
-         ( $debugging ? (CLEAN_AFTER_BUILD => 0) : () ));
+         LIBS => join(" ", qw(-lcrypto -lssl)),
+         ($class->full_debugging ? (OPTIMIZE => "-g") :
+          (OPTIMIZE => "-g -O2")),
+         CCFLAGS => join(" ", qw(-Wall -Wno-unused -Werror)),
+         ( $class->full_debugging ? (CLEAN_AFTER_BUILD => 0) : () ));
     uplevel 1, \&Inline::import, Inline => C => (_c_boilerplate . $c_code);
     return;
 }
@@ -388,6 +520,23 @@ Called when L</the "__END__" pragma> is seen.  Currently does nothing.
 
 sub compile_everything {}
 
+=item I<full_debugging>
+
+Returns true iff the environment variable C<FULL_DEBUGGING> is set.
+This causes the C code to be compiled without optimization, allowing
+gdb to dump symbols of static functions with only one call site (which
+comprises most of the C code in L<Crypt::OpenSSL::CA>).  Also, the
+temporary build files are left intact if C<FULL_DEBUGGING> is set.
+
+Developpers, please note that in the absence of C<FULL_DEBUGGING>, the
+default compiler flags are C<-g -O2>, still allowing for a range of
+debugging strategies.  C<FULL_DEBUGGING> should therefore only be set
+on a one-shot basis by developpers who have a specific need for it.
+
+=cut
+
+sub full_debugging { ! ! $ENV{FULL_DEBUGGING} }
+
 =item I<installed_version>
 
 Returns what the source code of this module will look like (with POD
@@ -435,8 +584,8 @@ Crypt::OpenSSL::CA
 
 =head1 DESCRIPTION
 
-This package simply loads the DLL that contains the part of
-L<Crypt::OpenSSL::CA> that is made of XS code. It is a stubbed-down
+This package simply loads the DLLs that contain the parts of
+L<Crypt::OpenSSL::CA> that are made of XS code. It is a stubbed-down
 version of the full-fledged I<Crypt::OpenSSL::CA::Inline::C> that
 replaces the real thing at module install time.
 
@@ -454,8 +603,6 @@ INSTALLED_VERSION
 
 =end this_pod_is_not_ours
 
-=back
-
 =head1 TODO
 
 Right now it is not possible to invoke
@@ -463,6 +610,10 @@ I<Crypt::OpenSSL::CA::Inline::C> several times from the same package,
 yet allowing that would be straightforward.  the immediate benefit of
 that would be to allow to intermingle POD and C code freely.
 
+=head1 SEE ALSO
+
+L<Inline::C>, L<perlxstut>, L<perlguts>, L<perlapi>.
+
 =cut
 
 require My::Tests::Below unless caller();
@@ -582,9 +733,60 @@ C_TEST
     leaks_SVs_ok { map { Foo->new($_) } (1..1000) };
 };
 
+{ package TestCRoutines; use Crypt::OpenSSL::CA::Inline::C <<"C_TEST"; }
+void argh() {
+    sslcroak("How are you %s", "gentlemen");
+}
+
+// For leak tests
+void ulp() {
+    char buf[1024];
+    memset(buf, (int) 'A', 1023);
+    buf[1023] = '\\0';
+    sslcroak(buf);
+}
+
+int test_parse_RFC3280_time(char* time) {
+    char* plainerr = NULL; char* sslerr = NULL;
+    ASN1_TIME* asn1time;
+    int retval;
+
+    asn1time = parse_RFC3280_time(time, &plainerr, &sslerr);
+    if (asn1time) {
+        retval = (asn1time->type == V_ASN1_GENERALIZEDTIME ? 1 : 0);
+        ASN1_TIME_free(asn1time);
+        return retval;
+    }
+    if (plainerr) { return -1; }
+    if (sslerr) { return -2; }
+    return -42;
+}
+
+void test_parse_RFC3280_time_or_croak(char* time) {
+    ASN1_TIME_free(parse_RFC3280_time_or_croak(time));
+}
+
+int test_parse_serial(char* serial) {
+    char* plainerr = NULL; char* sslerr = NULL;
+    ASN1_INTEGER* asn1serial;
+
+    asn1serial = parse_serial(serial, &plainerr, &sslerr);
+    if (asn1serial) { ASN1_INTEGER_free(asn1serial); return 0; }
+    if (plainerr) { return 1; }
+    if (sslerr) { return 2; }
+    return 42;
+}
+
+void test_parse_serial_or_croak(char* serial) {
+    ASN1_INTEGER_free(parse_serial_or_croak(serial));
+}
+
+C_TEST
+
+
 test "sslcroak()" => sub {
     # Implementation lifted from Crypt::OpenSSL::CA so as
-    # to sever the circular dependency in tests: 
+    # to sever the circular dependency in tests:
     sub Crypt::OpenSSL::CA::_sslcroak_callback {
         my ($key, $val) = @_;
         if ($key eq "-message") {
@@ -602,23 +804,8 @@ test "sslcroak()" => sub {
         }
     }
 
-    { package TestSslcroak; use Crypt::OpenSSL::CA::Inline::C <<"C_TEST"; }
-void argh() {
-    sslcroak("How are you %s", "gentlemen");
-}
-
-// For leak tests
-void ulp() {
-    char buf[1024];
-    memset(buf, (int) 'A', 1023);
-    buf[1023] = '\\0';
-    sslcroak(buf);
-}
-
-C_TEST
-
     eval {
-        TestSslcroak::argh;
+        TestCRoutines::argh;
         fail("Should have thrown");
     };
     is(ref($@), "Crypt::OpenSSL::CA::Error") or warn Dumper($@);
@@ -628,7 +815,7 @@ C_TEST
     errstack_empty_ok();
     use Net::SSLeay;
     is(Net::SSLeay::BIO_new_file("/no/such/file_", "r"), 0);
-    eval { TestSslcroak::argh; fail("should have thrown") };
+    eval { TestCRoutines::argh; fail("should have thrown") };
     is(ref($@), "Crypt::OpenSSL::CA::Error");
     is(ref($@->{-openssl}), "ARRAY");
     cmp_ok(scalar(@{$@->{-openssl}}), ">", 0);
@@ -639,8 +826,54 @@ C_TEST
 
     unless (cannot_check_bytes_leaks) {
         leaks_bytes_ok {
-            for(1..200) { eval { TestSslcroak::ulp }; }
+            for(1..200) { eval { TestCRoutines::ulp }; }
         };
     }
 };
 
+test "parse_RFC3280_time" => sub {
+    is(TestCRoutines::test_parse_RFC3280_time("20510103103442Z"), 1);
+    is(TestCRoutines::test_parse_RFC3280_time("19510103103442Z"), 0);
+    TestCRoutines::test_parse_RFC3280_time_or_croak("19510103103442Z");
+    eval {
+        TestCRoutines::test_parse_RFC3280_time_or_croak("0Z");
+        fail("Should have thrown");
+    };
+};
+
+skip_next_test "Memchmark needed" if cannot_check_bytes_leaks;
+test "parse_RFC3280_time_or_croak memory leaks" => sub {
+    leaks_bytes_ok {
+        for(1..10000) {
+            TestCRoutines::test_parse_RFC3280_time("portnawak");
+            TestCRoutines::test_parse_RFC3280_time("20510103103442Z");
+            TestCRoutines::test_parse_RFC3280_time("19510103103442Z");
+        }
+    };
+};
+
+test "parse_serial" => sub {
+    TestCRoutines::test_parse_serial_or_croak("0xdeadbeef1234");
+    pass;
+    eval {
+        TestCRoutines::test_parse_serial_or_croak("deadbeef1234");
+        fail("should have thrown");
+    };
+    ok(! ref($@), "Plain error expected");
+    unlike($@, qr/unknown/i, "proper internal error management");
+    is(TestCRoutines::test_parse_serial("0xdeadbeef1234"), 0);
+    is(TestCRoutines::test_parse_serial("deadbeef1234"), 1);
+};
+
+skip_next_test "Memchmark needed" if cannot_check_bytes_leaks;
+test "parse_serial memory leaks" => sub {
+    leaks_bytes_ok {
+        for(1..10000) {
+            TestCRoutines::test_parse_serial("0xdeadbeef1234");
+            TestCRoutines::test_parse_serial("deadbeef1234");
+        }
+    };
+};
+
+=cut
+
